@@ -7,12 +7,21 @@ import numpy as np
 
 def pose_infer_demo():
     # a = time.time()
-    img = 'eason.png' # 指定进行推理的图片路径
-    pose = wf(task='body')# ,checkpoint="rtmpose-m-80e511.onnx") # 实例化mmpose模型
+    img = 'pose2.jpg' # 指定进行推理的图片路径
+    img = cv2.imread(img)
 
-    result,img = pose.inference(data=img,img_type='pil') # 在CPU上进行推理
-    print(result)
-    pose.show(img)
+    det = wf(task='det_body_l')#,checkpoint="checkpoints/bodydetect_l.onnx")
+    pose = wf(task='pose_body_l')# ,checkpoint="rtmpose-l-19c9d1.onnx")# "rtmpose-m-80e511.onnx") # 实例化mmpose模型
+    
+    import time
+    a = time.time()
+    bbox = det.inference(data=img)
+    print(time.time()-a)
+    for i in bbox:
+        result,img = pose.inference(data=img,img_type='cv2',bbox=i) # 在CPU上进行推理
+        pose.show(img)
+    # cv2.imshow("img",img)
+    # cv2.waitKey(0)
     # pose.save(img,"pimg_ou.png")
     
     result = pose.format_output(lang="zh")
@@ -42,11 +51,11 @@ def video_infer_demo():
 def det_infer_demo():
     # a = time.time()
     from XEdu.hub import Workflow as wf
-    img = 'pose4.jpg' # 指定进行推理的图片路径
+    img = 'hand1.jpeg' # 指定进行推理的图片路径
 
-    det = wf(task='det_body')#,checkpoint='rtmdet-acc0de.onnx')# ,checkpoint="rtmpose-m-80e511.onnx") # 实例化mmpose模型
+    det = wf(task='det_hand')#,checkpoint='rtmdet-acc0de.onnx')# ,checkpoint="rtmpose-m-80e511.onnx") # 实例化mmpose模型
 
-    bboxs,im_ou = det.inference(data=img,img_type='cv2',thr=0.3,show=True) # 在CPU上进行推理
+    bboxs,im_ou = det.inference(data=img,img_type='pil',show=True) # 在CPU上进行推理
     # print(bboxs)
     # det.save(im_ou,"im_ou_d.jpg")
 
@@ -64,6 +73,7 @@ def hand_video_demo():
         if not ret:
             break
         keypoints,img =pose.inference(data=frame,img_type='cv2') # 在CPU上进行推理
+        
         cv2.imshow('video', img)
         if cv2.waitKey(1) & 0xFF == ord('q'):
             break    
@@ -71,29 +81,31 @@ def hand_video_demo():
     cv2.destroyAllWindows()
 
 def coco_det_demo():
-    img = 'pose1.jpg' # 指定进行推理的图片路径
-    det = wf(task='det_coco') # 实例化mmpose模型
+    img = 'coco.jpg' # 指定进行推理的图片路径
+    det = wf(task='det_coco_l')#,checkpoint="checkpoints/cocodetect_l.onnx") # 实例化mmpose模型
 
-    result,img = det.inference(data=img,img_type='cv2') # 在CPU上进行推理
+    result,img = det.inference(data=img,img_type='pil',thr=0.3) # 在CPU上进行推理
     det.show(img)
-    # det.save(img,"pimg_ou.png")
+    det.save(img,"pimg_ou.png")
     
-    re = det.format_output(lang="zh")
+    # re = det.format_output(lang="zh")
 
 def face_det_demo():
-    img = 'pose3.jpg' # 指定进行推理的图片路径
+    img = 'face2.jpeg' # 指定进行推理的图片路径
     # img = 'face2.jpeg' # 指定进行推理的图片路径
 
     det = wf(task='det_face' )
     face = wf(task="pose_face")
 
     result,img = det.inference(data=img,img_type='cv2') # 在CPU上进行推理
+    re =  det.format_output(lang="zh")
+
     det.show(img)
     for i in result:
         ky,img = face.inference(data=img, img_type="cv2",bbox=i)#,erase=False)
         face.show(img)
     
-    # re = face.format_output(lang="zh")
+    re =  det.format_output(lang="zh")
 
 def ocr_demo():
     img = 'ocr.jpg' # 指定进行推理的图片路径
@@ -193,10 +205,11 @@ def sup_demo():
     print(cos_sim)
 
 def style():
-    # for i in range(5):
-    #     style = wf(task='gen_style' ,style=i) # iris act 
-    #     result = style.inference(data="ele.jpg" ,img_type='cv2')
-    #     style.show(result)
+    for i in range(5):
+        style = wf(task='gen_style' ,style=i) # iris act 
+        result = style.inference(data="ele.jpg" ,img_type='cv2')
+        style.show(result)
+        style.save(result,"ele_{}.jpg".format(i))
     #     result,img = style.inference(data="ele.jpg" ,img_type='cv2')
     #     style.show(result)
     #     result,img = style.inference(data="ele.jpg" ,img_type='pil')
@@ -208,21 +221,47 @@ def style():
         # cv2.imshow("result",result)
         # cv2.waitKey(0)
         # style.show(img)
-    
-    style = wf(task='gen_style',style=0) 
-    result,img = style.inference(data="ele.jpg",img_type='pil')
-    
-    # style.show(result)
-    # 打开摄像头，实时风格迁移
-    cap = cv2.VideoCapture(0)
-    while cap.isOpened():
-        ret, frame = cap.read()
-        if not ret:
-            break
-        result,img = style.inference(data=frame,img_type='cv2')
-        cv2.imshow('video', img)
-        if cv2.waitKey(1) & 0xFF == ord('q'):
-            break
+    # styles = [ wf(task='gen_style',style=i) for i in range(5) ]
+    # # result,img = style.inference(data="ele.jpg",img_type='pil')
+    # import random
+    # # style.show(result)
+    # # 打开摄像头，实时风格迁移
+    # cap = cv2.VideoCapture(0)
+    # while cap.isOpened():
+    #     ret, frame = cap.read()
+    #     if not ret:
+    #         break
+    #     style = random.choice(styles)
+    #     result,img = style.inference(data=frame,img_type='cv2')
+    #     cv2.imshow('video', img)
+    #     if cv2.waitKey(1) & 0xFF == ord('q'):
+    #         break
+
+def qa_demo():
+    qa = wf(task='nlp_qa')# ,checkpoint="checkpoints/bertsquad-8s.onnx")  # /home/user/下载/chatgpt_项目/bertsquad-12-int8.onnx
+    # qa = wf(task='nlp_qa',checkpoint="/home/user/下载/chatgpt_项目/bertsquad-12-int8.onnx")
+    # 整理以下字符串
+    context_glm = """In life, we often encounter challenges that make us question our abilities and motivate us to give up. However, the secret to success lies in persistence. Only by persevering through difficult times can we unlock our true potential and achieve our goals.
+Michael Jordan, one of the greatest basketball players of all time, was once cut from his high school team. Instead of giving up, he practiced relentlessly and eventually earned a spot on the team. His perseverance and determination are what made him a legendary athlete.
+Another example of persistence is Elon Musk, the CEO of SpaceX and Tesla. He has faced countless setbacks in his entrepreneurial ventures, but he hasn't let them stop him. His vision for the future and relentless pursuit of innovation have made him a global sensation.
+In life, we will all face obstacles. It's how we respond to these challenges that defines us. Instead of surrendering to defeat, let's embrace the power of persistence and strive for greatness. Remember, success is not a destination but a journey, and the only way to achieve it is to keep moving forward.
+"""
+    q11 = "Who is Michael Jordan?"
+    q22 = "Who is Elon Musk?"
+    context = "In its early years, the new convention center failed to meet attendance and revenue expectations.[12] By 2002, many Silicon Valley businesses were choosing the much larger Moscone Center in San Francisco over the San Jose Convention Center due to the latter's limited space. A ballot measure to finance an expansion via a hotel tax failed to reach the required two-thirds majority to pass. In June 2005, Team San Jose built the South Hall, a $6.77 million, blue and white tent, adding 80,000 square feet (7,400 m2) of exhibit space"
+    q1 = "By what year many Silicon Valley businesses were choosing the Moscone Center?"
+    q2 = "how may votes did the ballot measure need?"
+    q3 = "how many square feet did the South Hall add?"
+    qa.load_context(context)
+    result = qa.inference(data=q11, context=context_glm)
+    result = qa.inference(data=q2)
+    result = qa.inference(data=q22, context=context_glm)
+    result = qa.inference(data=q3)
+
+    print(result)
+    res = qa.format_output(lang="en",show_context=False)
+    # result, r = qa.inference(data=q4)
+    # print(result)
 
 def sim():
     from onnxsim import simplify
@@ -231,8 +270,19 @@ def sim():
     model,_ = simplify(model)
     onnx.save(model, "gen_style_udnie.onnx")
 
+def drive_demo():
+    from XEdu.hub import Workflow as wf
+    drp = wf(task='drive_perception')# ,checkpoint="checkpoints/yolopv2_post_Nx3x192x320.onnx") 
+    # result = drp.inference(data="demo.jpg")
+
+    result,image = drp.inference(data="cat1.jpg",img_type='pil')
+    drp.format_output(lang="zh")
+
+    drp.show(image)
+    # drp.save(image,"demo_ou.jpg")
+
 if __name__ == "__main__":
-    # pose_infer_demo()
+    pose_infer_demo()
     # det_infer_demo()
     # video_infer_demo()
     # hand_video_demo()
@@ -245,7 +295,7 @@ if __name__ == "__main__":
     # basenn_demo()
     # cls_demo()
     # baseml_demo()
-    # sup_demo()
-    # whisper()
-    style()
+    # style()
+    # qa_demo()
+    # drive_demo()
 
