@@ -15,18 +15,25 @@ import pprint
 from openxlab.model import download
 import os
 from PIL import Image
+from  .qa_tokenize import *
+from .qa_squadprocess import *
 
 task_dict = {
         "pose_body17":"body17.onnx",
+        "pose_body17_l":"body17_l.onnx",
         "pose_body26":"body26.onnx",
         "pose_wholebody133":"whole133.onnx",
         "pose_face106":"face106.onnx",
         "pose_hand21":"hand21.onnx",
         "det_body":"bodydetect.onnx",
+        "det_body_l":"bodydetect_l.onnx",
         "det_coco":"cocodetect.onnx",
+        "det_coco_l":"cocodetect_l.onnx",
         "det_hand":"handdetect.onnx",
         "cls_imagenet":"imagenet1k.onnx",
         "gen_style":"gen_style_mosaic.onnx",
+        "nlp_qa":"nlp_qa.onnx",
+        "drive_perception":"drive_perception.onnx",
         "det_face":"...",
         "ocr":"...",
         "mmedu":"...",
@@ -54,6 +61,8 @@ class Workflow:
         self.task_dict = task_dict
         self.task_nick_name ={
             'body':"pose_body17",
+            'body_l':"pose_body17_l",
+            'pose_body_l':"pose_body17_l",
             "face":"pose_face106",
             "hand":"pose_hand21",
             "wholebody":"pose_wholebody133",
@@ -90,7 +99,13 @@ class Workflow:
             'l_blue':[0,0,128],
             'l_purple':[128,0,128],
         }
-        if checkpoint is None and self.task in['det_body','det_coco','pose_body17','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet','gen_style']: # 若不指定权重文件，则使用对应任务的默认模型
+        if self.task == 'nlp_qa':
+            self.answers = []
+            self.questions = []
+            self.contexts = []
+            self.doc = None
+            
+        if checkpoint is None and self.task in['det_body','det_body_l','det_coco','det_coco_l','pose_body17','pose_body17_l','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet','gen_style','nlp_qa','drive_perception']: # 若不指定权重文件，则使用对应任务的默认模型
             if self.task == 'gen_style':
                 self.style = kwargs.get('style',None)
                 if isinstance(self.style,int): # 按照索引选择风格
@@ -113,12 +128,15 @@ class Workflow:
                     os.mkdir(path)
                 model_name_map = {
                     "pose_body17":"17 body keypoints",
+                    "pose_body17_l":"17 body keypoints large model",
                     "pose_body26":"26 body keypoints",
                     "pose_wholebody133":"133 wholebody keypoints",
                     "pose_face106":"106 face keypoints",
                     "pose_hand21":"21 hand keypoints",
                     "det_body":"body detection",
+                    "det_body_l":"body detection large model",
                     "det_coco":"coco detection",
+                    "det_coco_l":"coco detection large model",
                     "det_hand":"hand detection",
                     "cls_imagenet":"imagenet1k classification",
                     "gen_style":"mosaic style transfer",
@@ -128,6 +146,9 @@ class Workflow:
                     "gen_style_udnie":"udnie style transfer",
                     "gen_style_pointilism":"pointilism style transfer",
                     "gen_style_custom":"custom style transfer",
+                    "nlp_qa":"question answering",
+                    "drive_perception":"drive perception",
+
                 }
                 # download('test12318/bert-english',model_name=model_name,output=path)
                 download(model_repo='xedu/hub-model', model_name=model_name_map[self.task],output=path)
@@ -141,9 +162,9 @@ class Workflow:
                 try :
                     from rapidocr_onnxruntime import RapidOCR
                 except:
-                    os.system("pip install rapidocr_onnxruntime")
+                    os.system("pip install rapidocr_onnxruntime==1.3.7")
                     from rapidocr_onnxruntime import RapidOCR
-                self.model = RapidOCR()
+                self.model = RapidOCR(text_score=0.2)
         elif self.task.lower() == "mmedu":
             if checkpoint is None:
                 print("请先指定通过MMEdu导出的onnx模型路径。")
@@ -168,12 +189,12 @@ class Workflow:
         print("模型加载成功！")
     
 
-    def inference(self,data=None,show=False,img_type=None,thr=0.3,bbox=None,target_class=None,erase=False,preprocess=None, postprocess=None):
+    def inference(self,data=None,show=False,img_type=None,thr=0.3,bbox=None,target_class=None,erase=False,preprocess=None, postprocess=None,**kwargs):
         self._get_img = img_type
         self.erase=erase
-        if self.task in ["pose_body17","pose_body26","pose_face106","pose_hand21","pose_wholebody133"]:
+        if self.task in ["pose_body17","pose_body26","pose_face106","pose_hand21","pose_wholebody133","pose_body17_l"]:
             return self._pose_infer(data,show,img_type,bbox)
-        elif self.task in['det_body','det_coco','det_hand']:
+        elif self.task in['det_body','det_coco','det_hand','det_body_l','det_coco_l']:
             return self._det_infer(data,show,img_type,thr,target_class)
         elif self.task in ['det_face']:
             return self._face_det_infer(data,show,img_type)
@@ -195,6 +216,170 @@ class Workflow:
             return self._baseml_infer(data,show,img_type)
         elif self.task in ['custom']:
             return self._custom_infer(data, preprocess, postprocess)
+        elif self.task in ['nlp_qa']:
+            return self._nlp_qa_infer(data,kwargs.get('context',None))
+        elif self.task in ['drive_perception']:
+            return self._drive_perception_infer(data,show,img_type,thr)
+    
+    def _drive_perception_infer(self,data=None,show=False,img_type=None,thr=0.3):
+        model_inputs = self.model.get_inputs()
+        self.input_name = model_inputs[0].name
+        self.input_names = [model_inputs[i].name for i in range(len(model_inputs))]
+        self.input_shape = model_inputs[0].shape
+        self.input_height = int(self.input_shape[2])
+        self.input_width = int(self.input_shape[3])
+        self.confThreshold = thr
+
+        if isinstance(data, str):
+            image = cv2.imread(data)
+        else:
+            image = data
+        image_width, image_height = image.shape[1], image.shape[0]
+        ratioh = image_height / self.input_height
+        ratiow = image_width / self.input_width
+
+        input_image = cv2.resize(image, dsize=(self.input_width, self.input_height))
+        input_image = cv2.cvtColor(input_image, cv2.COLOR_BGR2RGB)
+        input_image = input_image.transpose(2, 0, 1)
+        input_image = np.expand_dims(input_image, axis=0)
+        input_image = input_image.astype('float32')
+        input_image = input_image / 255.0
+
+        results = self.model.run(None, {self.input_name: input_image})
+        scores_ = results[2]
+        batchno_classid_y1x1y2x2 = results[3]
+        # Traffic Object Detection
+        bboxes, scores, class_ids = [], [], []
+        for score, batchno_classid_y1x1y2x2_ in zip(scores_, batchno_classid_y1x1y2x2):
+            if score < self.confThreshold:
+                continue
+
+            class_id = int(batchno_classid_y1x1y2x2_[1])
+            y1 = batchno_classid_y1x1y2x2_[2]
+            x1 = batchno_classid_y1x1y2x2_[3]
+            y2 = batchno_classid_y1x1y2x2_[4]
+            x2 = batchno_classid_y1x1y2x2_[5]
+            y1 = int(y1 * ratioh)
+            x1 = int(x1 * ratiow)
+            y2 = int(y2 * ratioh)
+            x2 = int(x2 * ratiow)
+
+            bboxes.append([x1, y1, x2, y2])
+            class_ids.append(class_id)
+            scores.append(score)
+
+        # Drivable Area Segmentation
+        drivable_area = np.squeeze(results[0], axis=0)
+        mask = np.argmax(drivable_area, axis=0).astype(np.uint8)
+        area_mask = cv2.resize(mask, (image_width, image_height), interpolation=cv2.INTER_NEAREST)
+
+        # Lane Line
+        lane_line = np.squeeze(results[1])
+        mask = np.where(lane_line > 0.5, 1, 0).astype(np.uint8)
+        lane_mask = cv2.resize(mask, (image_width, image_height), interpolation=cv2.INTER_NEAREST)
+        # 将x1y1x2y2转化为x1y1w,h
+        if len(bboxes) != 0:
+            bboxes = np.array(bboxes)
+            bboxes[:,2] = bboxes[:,2] - bboxes[:,0]
+            bboxes[:,3] = bboxes[:,3] - bboxes[:,1]
+        else:
+            bboxes = np.array(bboxes)
+        results = [bboxes, lane_mask, area_mask] # , scores]
+        self.bboxes = bboxes
+        self.lane_mask = lane_mask
+        self.area_mask = area_mask
+        self.scores = scores
+        if img_type:
+            # 绘制查看效果
+            self._get_img = img_type
+            # lane_mask, area_mask, bboxes = results
+            if img_type =='pil':
+                image = cv2.cvtColor(image,cv2.COLOR_BGR2RGB)
+            
+            for bbox, score in zip(bboxes, scores):
+                x1, y1 = int(bbox[0]), int(bbox[1])
+                x2, y2 = int(bbox[2] + bbox[0]), int(bbox[3]+bbox[1])
+
+                cv2.rectangle(image, (x1, y1), (x2, y2), (0, 0, 255), 2)
+                # cv2.putText(image, '%s:%.2f' % (self.classes[class_id-1], score), (x1, y1 - 5), 0,
+                #            0.7, (0, 255, 0), 2)
+            image[area_mask==1] = [0, 255, 0]
+            image[lane_mask==1] = [255, 0, 0]
+            if show:
+                self.show(image)
+            return results, image
+        return results
+
+    def load_context(self,context):
+        self.context = context
+        self.doc = read_squad_examples_context(input_data=context)
+        return self.doc 
+
+
+    def _nlp_qa_infer(self,data=None,context=None):
+        if context is not None:
+            self.contexts.append(context)
+            doc = read_squad_examples_context(input_data=context)
+        else:
+            if self.doc is None:
+                print("Please load_context before inference !")
+                return
+            doc = self.doc
+            self.contexts.append(self.context)
+
+
+        if os.path.splitext(data)[1] == '.json':
+            eval_examples = read_squad_examples_file(input_file=data)
+            self.questions = []
+            with open(data, "r") as f:
+                input_data = json.load(f)["data"]
+            print("inout",input_data)
+            for idx, entry in enumerate(input_data):
+                for paragraph in entry["paragraphs"]:
+                    for qa in paragraph["qas"]:
+                        qas_id = qa["id"]
+                        question_text = qa["question"]
+                        print("question_text",question_text)
+                        self.questions.append(question_text)
+        else:   
+            self.questions.append(data)
+            eval_examples = [squad_example(qas_id="1",question_text=data,doc_tokens=doc)]
+        # eval_examples = read_squad_examples_file(input_file=predict_file)
+
+        max_seq_length = 256
+        doc_stride = 128
+        max_query_length = 64
+        batch_size = 1
+        n_best_size = 20
+        max_answer_length = 64
+
+        vocab_file = os.path.join(os.path.dirname(__file__), 'vocab.txt')
+        # vocab_file = 'vocab.txt'
+        tokenizer = FullTokenizer(vocab_file=vocab_file, do_lower_case=True)
+
+        input_ids, input_mask, segment_ids, extra_data = convert_examples_to_features(eval_examples, tokenizer, max_seq_length, doc_stride, max_query_length)
+
+        n = len(input_ids)
+        bs = batch_size
+        all_results = []
+        for idx in range(0, n):
+            data = {"unique_ids_raw_output___9:0": np.array([1], dtype=np.int64),
+                    "input_ids:0": input_ids[idx:idx+bs],
+                    "input_mask:0": input_mask[idx:idx+bs],
+                    "segment_ids:0": segment_ids[idx:idx+bs]}
+            result = self.model.run(["unique_ids:0","unstack:0", "unstack:1"], data)
+            in_batch = result[1].shape[0]
+            start_logits = [float(x) for x in result[1][0].flat]
+            end_logits = [float(x) for x in result[2][0].flat]
+            for i in range(0, in_batch):
+                unique_id = len(all_results)
+                all_results.append(RawResult(unique_id=unique_id, start_logits=start_logits, end_logits=end_logits))
+
+        res, answer = write_predictions(eval_examples, extra_data, all_results,
+                        n_best_size, max_answer_length,True)
+        res = dict(res)["1"]
+        self.answers.append(answer)
+        return res
     
     def _baseml_infer(self,data=None, show=None, img_type=None):
         self.baseml_res = self.model.predict(data)
@@ -459,7 +644,8 @@ class Workflow:
             ex_bbox = [bbox[0],bbox[1],(bbox[0]+bbox[2]),(bbox[1]+bbox[3])] 
             self.bboxs.append(ex_bbox)
         self.scores = []
-        self.classes = ["face" for i in range(len(self.bboxs))]
+        # self.classes = ["face" for i in range(len(self.bboxs))]
+        self.classes = None
 
         if get_img:
             if get_img =='cv2':
@@ -535,13 +721,24 @@ class Workflow:
         if get_img:
             if get_img =='cv2':
                 for i,[a,b,c,d] in enumerate(self.bboxs):
-                    if self.task == 'det_coco':
-                        cv2.putText(image, str(self.classes[i]), (int(a),int(b)),cv2.FONT_HERSHEY_COMPLEX,1,(0,0,255), 2)
-                    cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,255),2)
+                    # cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,255),2)
+                    if self.task in ['det_coco','det_coco_l']:
+                        # cv2.putText(image, str(self.classes[i]), (int(a),int(b)),cv2.FONT_HERSHEY_COMPLEX,1,(0,0,255), 2)
+                        # (w,h), _ = cv2.getTextSize(str(self.classes[i]),cv2.FONT_HERSHEY_COMPLEX,1,2)
+                        # print(w,h)
+                        image = pil_draw(image, bbox1=(int(a), int(b), int(c), int(d)), label1=str(self.classes[i]))
+                    # cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,0),2)
+                    else:
+                        cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,255),2)
+
+
             elif get_img =='pil':
                 # image = cv2.cvtcolor(image,cv2.BGR2RGB)
-                for [a,b,c,d] in self.bboxs:
-                    cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,255),2)
+                for i, [a,b,c,d] in enumerate(self.bboxs):
+                    if self.task in ['det_coco','det_coco_l']:
+                        image = pil_draw(image, bbox1=(int(a), int(b), int(c), int(d)), label1=str(self.classes[i]))
+                    else:
+                        cv2.rectangle(image, (int(a),int(b)),(int(c),int(d)),(0,0,255),2)
                 image = cv2.cvtColor(image,cv2.COLOR_BGR2RGB)
             if show:
                 self.show(image)
@@ -588,6 +785,7 @@ class Workflow:
     
     def _get_image(self):
         sketch = {}
+        ratio = 1
         if self.task == 'pose_hand21':
             sketch = {
                 'red':[[0,1],[1,2],[2,3],[3,4]],
@@ -605,7 +803,7 @@ class Workflow:
                 'blue':[[11,13],[13,15],[15,20],[15,22],[15,24]],
                 'purple':[[18,5],[5,7],[7,9]],
             }
-        elif self.task =='pose_body17':
+        elif self.task in['pose_body17','pose_body17_l']:
             sketch = {
                 'red':[[0,1],[1,2],[2,0],[2,4],[1,3],[4,6],[3,5]],
                 'orange':[[10,8],[8,6]],
@@ -624,20 +822,26 @@ class Workflow:
                 'blue':[[11,13],[13,15]],
                 'purple':[[5,7],[9,7]],
             }
-
+            ratio = 0.3
+        elif self.task == 'pose_face106':
+            ratio = 0.3
         if isinstance(self.data,str):
             img = cv2.imread((self.data))
         else:
             img = self.data
-        # h,w,c = img.shape
-        
+        h,w,c = img.shape
+        sketch_scale = max(1,(min(h,w)/ 100))
+
+        point_scale = max(1,int(sketch_scale*ratio))
+
+        # sketch 
         for color in sketch.keys():
             # print(color,sketch[color])
             for [fx,fy] in sketch[color]:
                 # plt.plot([self.keypoints[0][fx][0],self.keypoints[0][fy][0]],[self.keypoints[0][fx][1],self.keypoints[0][fy][1]],color=color)
-                cv2.line(img, (int(self.keypoints[0][fx][0]),int(self.keypoints[0][fx][1])),(int(self.keypoints[0][fy][0]),int(self.keypoints[0][fy][1])),color=self.color_dict[color],thickness=5)
-
+                cv2.line(img, (int(self.keypoints[0][fx][0]),int(self.keypoints[0][fx][1])),(int(self.keypoints[0][fy][0]),int(self.keypoints[0][fy][1])),color=self.color_dict[color],thickness=max(1,int(sketch_scale/2)))
 
+        # keypoints
         for j in range(self.keypoints.shape[0]):
             for i in range(self.keypoints.shape[1]):
                 # plt.scatter(self.keypoints[j][i][0],self.keypoints[j][i][1],c='b',s=10)
@@ -645,9 +849,9 @@ class Workflow:
                 if self.bbox is not None and self.erase:
                     sx1,sy1,sx2,sy2 = self.bbox # 
                     if sx1<x1<sx2 and sy1<y1<sy2 :
-                        cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=2,color=[0,255,0])
+                        cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=max(1,int(point_scale/2)),color=[0,255,255],thickness=int(point_scale))
                 else:
-                    cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=5,color=[0,255,255],thickness=10)
+                    cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=max(1,int(point_scale/2)),color=[0,255,255],thickness=int(point_scale))
 
         if self._get_img == 'cv2':
             return img
@@ -672,8 +876,12 @@ class Workflow:
     def save(self,img,save_path): # check
         if self._get_img == 'cv2':
             # img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+            if isinstance(img,tuple):
+                img = img[1]
             cv2.imwrite(save_path,img)
         elif self._get_img.lower() == 'pil':
+            if isinstance(img,tuple):
+                img = img[1]
             img = img
             a = Image.fromarray(img)
             a.save(save_path)
@@ -683,9 +891,9 @@ class Workflow:
             # plt.margins(0,0)
             # plt.savefig(save_path)
     
-    def format_output(self,lang='zh'): # check
+    def format_output(self,lang='zh',**kwargs): # check
         language = lang
-        if self.task in ["pose_body17","pose_body26","pose_face106","pose_hand21","pose_wholebody133"]:
+        if self.task in ["pose_body17","pose_body26","pose_face106","pose_hand21","pose_wholebody133","pose_body17_l"]:
             formalize_keys = {
                 "zh":["关键点坐标","分数"],
                 "en":["keypoints","scores"],
@@ -699,7 +907,7 @@ class Workflow:
             }
             # formalize_result = json.dumps(formalize_result, indent=4,sort_keys=False)
             pprint.pprint(formalize_result,sort_dicts=False)
-        elif self.task in['det_body','det_coco','det_hand']:
+        elif self.task in['det_body','det_coco','det_hand','det_body_l','det_coco_l',]:
             formalize_keys = {
                 "zh":["检测框","分数","类别"],
                 "en":['bounding boxes',"scores","class"],
@@ -707,7 +915,7 @@ class Workflow:
                 "de":["Begrenzungsrahmen","Partituren","Klassen"],
                 "fr":["cadres de délimitation","partitions","Des classes"]
             }
-            if self.task == 'cocodetect' and self.classes is not None:
+            if self.task == 'det_coco' and self.classes is not None:
                 formalize_result = {
                     formalize_keys[language][0]:self.bboxs,
                     formalize_keys[language][1]:self.scores,
@@ -773,7 +981,7 @@ class Workflow:
             pprint.pprint(formalize_result,sort_dicts=False)
         elif self.task in ['baseml']:
             formalize_keys = {
-                "zh":["预测结果","分数"],
+                "zh":["预测值","分数"],
                 "en":["prediction","scores"],
                 "ru":["прогноз","баллы"],
                 "de":["Vorhersage","Partituren"],
@@ -800,8 +1008,73 @@ class Workflow:
             pprint.pprint(formalize_result,sort_dicts=False)
         elif self.task in ['gen_style_mosaic','gen_style_candy','gen_style_rain-princess','gen_style_udnie','gen_style_pointilism','gen_style_custom']:
             formalize_result = self.style_res
+        elif self.task in ['nlp_qa']:
+            formalize_keys = {
+                "zh":["问题","回答","文本","分数","上下文"],
+                "en":["question","answer","text","score","context"],
+                "ru":["вопрос","ответ","текст","баллы","контекст"],
+                "de":["Frage","Antwort","Text","Partituren","Kontext"],
+                "fr":["question","réponse","texte","partitions","contexte"]
+            }
+            formalize_results = []
+            for i, question in enumerate(self.questions):
+                answer = self.answers[i]["1"]
+                answer = [{formalize_keys[language][2]:i['text'],formalize_keys[language][3]:i['probability']}  for i in answer if i['probability'] >0.01]
+                if kwargs.get('show_context',False):
+                    formalize_result = {    
+                        formalize_keys[language][4]:self.contexts[i],
+                        formalize_keys[language][0]:question,
+                        formalize_keys[language][1]:answer,
+                    }
+                else:
+                    formalize_result = {    
+                        formalize_keys[language][0]:question,
+                        formalize_keys[language][1]:answer,
+                    }
+                formalize_results.append(formalize_result)
+            pprint.pprint(formalize_results,sort_dicts=False)
+        elif self.task in ['drive_perception']:
+            formalize_keys = {  
+                "zh":["检测框","分数","车道线掩码","可行驶区域掩码"],
+                "en":['bounding boxes',"scores","lane line mask","drivable area mask"],
+                "ru":["ограничивающие рамки","баллы","маска линии полосы","маска области движения"],
+                "de":["Begrenzungsrahmen","Partituren","Fahrspurmasken","Fahrbare Bereichsmaske"],
+                "fr":["cadres de délimitation","partitions","masque de ligne de voie","masque de zone praticable"]
+            }
+            formalize_result = {
+                formalize_keys[language][0]:self.bboxes.tolist(),
+                formalize_keys[language][1]:np.squeeze(self.scores).tolist(),
+                formalize_keys[language][2]:self.lane_mask,
+                formalize_keys[language][3]:self.area_mask,
+            }
+            pprint.pprint(formalize_result,sort_dicts=False)
+
         return formalize_result
 
+from PIL import Image, ImageDraw, ImageFont
+def pil_draw(img, bbox1, label1):
+    # 1.读取图片
+    im = Image.fromarray(img)
+    font_path = os.path.join(os.path.join(os.path.dirname(__file__), 'font'),'FZYTK.TTF')
+    # font_path = os.path.join(os.getcwd(), 'font/FZYTK.TTF')
+    font = ImageFont.truetype(font=font_path, size=np.floor(1.5e-2 * np.shape(im)[1] + 15).astype('int32'))
+
+    draw = ImageDraw.Draw(im)
+    # 获取label长宽
+    label_size1 = draw.textsize(label1, font)
+    # label_size2 = draw.textsize(label2, font)
+
+    # 设置label起点
+    text_origin1 = np.array([bbox1[0], bbox1[1] - label_size1[1]])
+    # text_origin2 = np.array([bbox2[0], bbox2[1] - label_size2[1]])
+
+    # 绘制矩形框，加入label文本
+    draw.rectangle([bbox1[0], bbox1[1], bbox1[2], bbox1[3]],outline='red',width=2)
+    draw.rectangle([tuple(text_origin1), tuple(text_origin1 + label_size1)], fill='red')
+    draw.text(text_origin1, str(label1), fill=(255, 255, 255), font=font)
+
+    return np.asarray(im)
+    
 
 def bbox_xyxy2cs(bbox: np.ndarray,
                  padding: float = 1.) -> Tuple[np.ndarray, np.ndarray]:
